@@ -17,7 +17,20 @@
  *
  */
 
+#[cfg(not(has_std))]
+pub(crate) mod std {
+    pub use core::*;
+    pub mod alloc {
+        pub use ::alloc::*;
+    }
+    pub mod collections {
+        pub use ::alloc::collections::*;
+    }
+    pub use ::alloc::vec;
+}
+
 use std::cmp::{Eq, Ord};
+#[cfg(has_std)]
 use std::collections::hash_map::RandomState;
 use std::hash::Hash;
 use std::iter::*;
@@ -43,6 +56,7 @@ where
     }
 }
 
+#[cfg(has_std)]
 pub struct IterMut<'a, I: 'a, P: 'a, H: 'a = RandomState>
 where
     I: Hash + Eq,
@@ -52,6 +66,16 @@ where
     pos: usize,
 }
 
+#[cfg(not(has_std))]
+pub struct IterMut<'a, I: 'a, P: 'a, H: 'a>
+where
+    I: Hash + Eq,
+    P: Ord,
+{
+    pq: &'a mut PriorityQueue<I, P, H>,
+    pos: usize,
+}
+
 impl<'a, I: 'a, P: 'a, H: 'a> IterMut<'a, I, P, H>
 where
     I: Hash + Eq,
@@ -109,6 +133,7 @@ where
     }
 }
 
+#[cfg(has_std)]
 pub struct IntoSortedIter<I, P, H = RandomState>
 where
     I: Hash + Eq,
@@ -117,6 +142,15 @@ where
     pub(crate) pq: PriorityQueue<I, P, H>,
 }
 
+#[cfg(not(has_std))]
+pub struct IntoSortedIter<I, P, H>
+where
+    I: Hash + Eq,
+    P: Ord,
+{
+    pub(crate) pq: PriorityQueue<I, P, H>,
+}
+
 impl<I, P, H> Iterator for IntoSortedIter<I, P, H>
 where
     I: Hash + Eq,
